/************************************************************************/
/*                                                                      */
/*  This example demonstrates an acquisition using the Equivalent Time  */
/*  Sampling technique.  ETS can be used on an E-Series DAQ device to   */
/*  achieve an effective acquisition rate of up to 20 MHz.  See the     */
/*  function  panel help for the Trigger Type parameter of              */
/*  AIAcquireTriggeredWaveforms() for details.  The graph shows both    */
/*  the ETS waveform and the "internal" waveform that was sampled using */
/*  internal timing so that you can compare them.                       */
/*                                                                      */
/*  For ETS to work, the waveform must be periodic.                     */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <utility.h>
#include <ansi_c.h>          
#include <userint.h>
#include <analysis.h>
#include "easyio.h"
#include "ai_ets.h"

static  int     AIRestartDAQ(void);
static  int     AIReadDAQ(void);
static  void    AIDisplayError (short error);
static  void    SetAutoTrigger (void);

static int              AIpanelHandle = 0;
static unsigned long    AItaskID = 0;
static double         * AIwaveforms = 0;
static int              AIhelpHandle;
static short            AIdevice;
static long             AIbacklog;
static unsigned long    AInumChannels;
static unsigned long    AInumScans = 1000;
static char             AIchannelString[100];
static double           upper, lower, AIscanRate;
static double           AIetsScanRate = 20e+6;
static double           actualAIscanRate = 20e+6;;
static double           actualAIetsScanRate;
static unsigned short   AIreadMode;
static int              displayOverWriteOccurred = 0;

static long             preTriggerScans;
static char             triggerSource[30];
static unsigned short   trigger_edge_slope;
static unsigned short   triggerType = NO_TRIGGER;
static double           triggerLevel;

static int              inTriggerMode = 0;
static int              needToAutoTrigger = 1;
static int              quitPressed = 0;

static int              ColorArray[8] = {VAL_GREEN, VAL_YELLOW, VAL_CYAN, VAL_WHITE, VAL_BLUE, VAL_MAGENTA, VAL_GRAY, VAL_RED};

static char             projDir [MAX_PATHNAME_LEN];
static char             triggerFileName [MAX_PATHNAME_LEN];
static int              previousBOLEstate;
static double           etsTimeLimit;
static short            etsError, analogTrigError;


int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    AIpanelHandle = LoadPanel (0, "ai_ets.uir", AI_PANEL);
    AIhelpHandle = LoadPanel (0, "ai_ets.uir", AI_HELP);
    
    AIDeviceCallback (AIpanelHandle, AI_PANEL_DEVICE, EVENT_COMMIT,
        0, 0, 0);
    AILimitCallback (AIpanelHandle, AI_PANEL_UPPER, EVENT_VAL_CHANGED,
        0, 0, 0);
    AILimitCallback (AIpanelHandle, AI_PANEL_LOWER, EVENT_VAL_CHANGED,
        0, 0, 0);
    AIScanRateCallback (AIpanelHandle, AI_PANEL_SCANRATE, EVENT_VAL_CHANGED,
        0, 0, 0);
    AIScanRateCallback (AIpanelHandle, AI_PANEL_ETS_SCANRATE, EVENT_VAL_CHANGED,
        0, 0, 0);
    AINumScansCallback (AIpanelHandle, AI_PANEL_NUMSCANS, EVENT_VAL_CHANGED,
        0, 0, 0);
    AIChannelCallback (AIpanelHandle, AI_PANEL_CHANNEL, EVENT_COMMIT,
        0, 0, 0);
    AIetsTimeLimitCallback (AIpanelHandle, AI_PANEL_ETS_TIME_LIMIT, EVENT_COMMIT,
        0, 0, 0);
    
    DisplayPanel (AIpanelHandle);
    RunUserInterface ();

    return 0;
}



int CVICALLBACK AIDeviceCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (AIpanelHandle, AI_PANEL_DEVICE, &AIdevice);
            AIChannelCallback (AIpanelHandle, AI_PANEL_CHANNEL, EVENT_COMMIT,
                0, 0, 0);

            needToAutoTrigger = 1;
            break;
    }
    return 0;
}

int CVICALLBACK AIChannelCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int error;

    
    switch (event) {
        case EVENT_RIGHT_CLICK:
            DisplayPanel(AIhelpHandle);
            break;
        case EVENT_COMMIT:
            GetCtrlVal (AIpanelHandle, AI_PANEL_CHANNEL, AIchannelString);
            
            previousBOLEstate = GetBreakOnLibraryErrors ();
            if (previousBOLEstate)
                DisableBreakOnLibraryErrors();

            error = GetNumChannels (AIdevice, AIchannelString, ANALOG_INPUT, 
                                &AInumChannels);
            AIDisplayError (error);
            
            error = GetChannelNameFromIndex (AIdevice, AIchannelString, 0,
                                                   ANALOG_INPUT, triggerSource);
            if (previousBOLEstate)
                EnableBreakOnLibraryErrors();
            
            AIDisplayError (error);
            
            free (AIwaveforms);
            AIwaveforms = malloc (AInumScans * AInumChannels * sizeof (double));
            if (AIwaveforms == 0)
            {
                error = memFullErr;
                AIDisplayError (error);
            }
            else
            {
                memset (AIwaveforms, 0, AInumScans * AInumChannels * sizeof (double));
                needToAutoTrigger = 1;
            }
            break;
    }
    return 0;
}

int CVICALLBACK AIChannelRingCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    
    char  * str;
    switch (event) {
        static int index;
        static int length;
        case EVENT_COMMIT:
            
            GetCtrlIndex (AIpanelHandle, AI_PANEL_CHANNELRING, &index);
            GetValueLengthFromIndex (AIpanelHandle, AI_PANEL_CHANNELRING, index,
                                     &length);
            str = (char *) malloc (length+1);
            GetCtrlVal (AIpanelHandle, AI_PANEL_CHANNELRING, str);
            ResetTextBox (AIpanelHandle, AI_PANEL_CHANNEL, str);
            AIChannelCallback (AIpanelHandle, AI_PANEL_CHANNEL, EVENT_COMMIT,
                0, 0, 0);
            free (str);
            
            break;
    }
    return 0;
}


int CVICALLBACK AILimitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{                             
    switch (event) {
        case EVENT_VAL_CHANGED:
            switch (control)
            {
                case AI_PANEL_UPPER:
                    GetCtrlVal (AIpanelHandle, AI_PANEL_UPPER, &upper);
                    if (upper <= lower)
                    {
                        upper = lower + 1.0;
                        SetCtrlVal (AIpanelHandle, AI_PANEL_UPPER, upper);
                    }
                
                break;
                case AI_PANEL_LOWER:
                    GetCtrlVal (AIpanelHandle, AI_PANEL_LOWER, &lower);
                    if (upper <= lower)
                    {
                        lower = upper - 1.0;
                        SetCtrlVal (AIpanelHandle, AI_PANEL_LOWER, lower);
                    }
                
                break;
            }
            
            SetAxisRange (AIpanelHandle, AI_PANEL_GRAPH, VAL_NO_CHANGE,
                              0.0, 1.0, VAL_MANUAL, lower, upper);
            SetCtrlAttribute (AIpanelHandle, AI_PANEL_TRIGGER_LEVEL,
                              ATTR_MAX_VALUE, upper);
            SetCtrlAttribute (AIpanelHandle, AI_PANEL_TRIGGER_LEVEL,
                              ATTR_MIN_VALUE, lower);
            
            break;
    }
    return 0;
}


int CVICALLBACK AIScanRateCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)

{
    switch (event) {
        case EVENT_VAL_CHANGED:
            switch (control) {
                case AI_PANEL_SCANRATE:
                    GetCtrlVal (AIpanelHandle, AI_PANEL_SCANRATE, &AIscanRate);
                
                break;
                case AI_PANEL_ETS_SCANRATE:
                    GetCtrlVal (AIpanelHandle, AI_PANEL_ETS_SCANRATE, &AIetsScanRate);
                
                break;
            }
        break;
    }
    return 0;
}

int CVICALLBACK AINumScansCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_VAL_CHANGED:
            if (callbackData)
                AInumScans =  * (unsigned long *) callbackData;
            else
                GetCtrlVal (AIpanelHandle, AI_PANEL_NUMSCANS, &AInumScans);
            free (AIwaveforms);
            AIwaveforms = malloc (AInumScans * AInumChannels * sizeof (double));
            if (AIwaveforms != 0)
            {
                memset (AIwaveforms, 0, AInumScans * AInumChannels * sizeof (double));
            }
            SetAxisRange (AIpanelHandle, AI_PANEL_GRAPH, VAL_MANUAL, 0.0,
                          (AInumScans)/actualAIetsScanRate, VAL_NO_CHANGE, 0.0, 1.0);
            break;
    }
    return 0;
}


int CVICALLBACK AICloseHelpCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            HidePanel (AIhelpHandle);
            break;
    }
    return 0;
}

int CVICALLBACK AITimerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int i;
    int error;
    int numInternalScans;
    static double lastETSscanRate;
    
    switch (event) {
        case EVENT_TIMER_TICK:
            
            if (needToAutoTrigger)
                 SetAutoTrigger();
                 
            
            lastETSscanRate = actualAIetsScanRate;
            
            previousBOLEstate = GetBreakOnLibraryErrors ();
            if (previousBOLEstate)
                DisableBreakOnLibraryErrors();

            etsError = AIAcquireTriggeredWaveforms (AIdevice, AIchannelString, AInumScans,
                                           AIetsScanRate, upper, lower,
                                           &actualAIetsScanRate, ETS_TRIGGER, 1, triggerLevel, "", 0,
                                           etsTimeLimit, GROUP_BY_CHANNEL, AIwaveforms);
            if (previousBOLEstate)
                EnableBreakOnLibraryErrors();
            
            AIDisplayError (etsError);

            SetCtrlVal (AIpanelHandle, AI_PANEL_ACTUAL_ETS_SCAN_RATE,
                        actualAIetsScanRate);
            
            SetCtrlAttribute (AIpanelHandle, AI_PANEL_GRAPH, 
                                ATTR_REFRESH_GRAPH, 0);

            error = DeleteGraphPlot (AIpanelHandle, AI_PANEL_GRAPH, -1,
                                 VAL_DELAYED_DRAW);
            
            if (lastETSscanRate != actualAIetsScanRate)
            {
                SetAxisRange (AIpanelHandle, AI_PANEL_GRAPH, VAL_MANUAL, 0.0,
                          (AInumScans)/actualAIetsScanRate, VAL_NO_CHANGE, 0.0, 1.0);
            }
            
            for (i=0; i<AInumChannels; i++)
            {
                if (i<8)
                {
                    PlotWaveform (AIpanelHandle, AI_PANEL_GRAPH,
                                  &AIwaveforms[AInumScans*i], AInumScans, VAL_DOUBLE, 1.0,
                                  0.0, 0.0, 1.0/actualAIetsScanRate, VAL_THIN_LINE,
                                  VAL_EMPTY_SQUARE, VAL_SOLID, 1, ColorArray[i]);
                }
                else
                {
                    PlotWaveform (AIpanelHandle, AI_PANEL_GRAPH,
                                  &AIwaveforms[AInumScans*i], AInumScans, VAL_DOUBLE, 1.0,
                                  0.0, 0.0, 1.0/actualAIetsScanRate, VAL_THIN_LINE,
                                  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
                }
            }

            ProcessSystemEvents();
            
            if (quitPressed)
                return 0;

            if (needToAutoTrigger)
                 SetAutoTrigger();
                 
            numInternalScans = AInumScans*AIscanRate/actualAIetsScanRate + 2;
            numInternalScans = AInumScans < numInternalScans ? AInumScans:numInternalScans;
            numInternalScans = 3 > numInternalScans ? 3:numInternalScans;
            
            if (numInternalScans > AInumScans)
                AINumScansCallback (AIpanelHandle, AI_PANEL_NUMSCANS, EVENT_VAL_CHANGED,
                    &numInternalScans, 0, 0);
            
            previousBOLEstate = GetBreakOnLibraryErrors ();
            if (previousBOLEstate)
                DisableBreakOnLibraryErrors();
            
            analogTrigError = AIAcquireTriggeredWaveforms (AIdevice, AIchannelString, 
                                                 numInternalScans,
                                                 AIscanRate, upper, lower,
                                                 &actualAIscanRate,
                                                 HW_ANALOG_TRIGGER, 1,
                                                 triggerLevel, "", 0, -1.0,
                                                 GROUP_BY_CHANNEL, AIwaveforms);
            if (previousBOLEstate)
                EnableBreakOnLibraryErrors();
            
            AIDisplayError (analogTrigError);
            
            SetCtrlVal (AIpanelHandle, AI_PANEL_ACTUAL_SCAN_RATE,
                        actualAIscanRate);

            for (i=0; i<AInumChannels; i++)
            {
                if (i<6)
                {
                    PlotWaveform (AIpanelHandle, AI_PANEL_GRAPH, &AIwaveforms[numInternalScans*i],
                                  numInternalScans, VAL_DOUBLE, 1.0, 0.0, 0.0,
                                  1.0/actualAIscanRate, VAL_CONNECTED_POINTS,
                                  VAL_SOLID_CIRCLE, VAL_SOLID, 1, ColorArray[i+2]);
                }
                else
                {
                    PlotWaveform (AIpanelHandle, AI_PANEL_GRAPH,
                                  &AIwaveforms[numInternalScans*i], numInternalScans, VAL_DOUBLE, 1.0,
                                  0.0, 0.0, 1.0/actualAIscanRate, VAL_CONNECTED_POINTS,
                                  VAL_SOLID_CIRCLE, VAL_SOLID, 1, VAL_RED);
                }
            }
            
            SetCtrlAttribute (AIpanelHandle, AI_PANEL_GRAPH, 
                                ATTR_REFRESH_GRAPH, 1);
            needToAutoTrigger = 0;
            break;
    }
    return 0;
}  

int CVICALLBACK TriggerLevelCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_VAL_CHANGED:
            SetCtrlAttribute (AIpanelHandle, AI_PANEL_TIMER, ATTR_INTERVAL, 0.3);
            GetCtrlVal (AIpanelHandle, AI_PANEL_TRIGGER_LEVEL, &triggerLevel);
            break;
        case EVENT_COMMIT:
            SetCtrlAttribute (AIpanelHandle, AI_PANEL_TIMER, ATTR_INTERVAL, 0.0);
            break;
    }
    return 0;
}

static void SetAutoTrigger ()
{
    double min;
    double max;
    int dummy, error;
    double highLimit, lowLimit;
    
    previousBOLEstate = GetBreakOnLibraryErrors ();
    if (previousBOLEstate)
        DisableBreakOnLibraryErrors();
            
    error = GetAILimitsOfChannel (AIdevice, AIchannelString, triggerSource,
                                  upper, lower, &highLimit, &lowLimit);

    error = AIAcquireWaveforms (AIdevice, triggerSource, AInumScans, AIscanRate,
                          highLimit, lowLimit, &actualAIscanRate,
                          GROUP_BY_CHANNEL, AIwaveforms);
    
    if (previousBOLEstate)
        EnableBreakOnLibraryErrors();

    if (error)
        return;
    
    MaxMin1D (AIwaveforms, AInumScans, &max, &dummy, &min, &dummy);
    triggerLevel = (max + min)/2.0;
    SetCtrlVal (AIpanelHandle, AI_PANEL_TRIGGER_LEVEL, triggerLevel);
}

int CVICALLBACK AIetsTimeLimitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (AIpanelHandle, AI_PANEL_ETS_TIME_LIMIT, &etsTimeLimit);
            break;
    }
    return 0;
}

int CVICALLBACK QuitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            free (AIwaveforms);
            AIwaveforms = 0;
            
            SetCtrlAttribute (AIpanelHandle, AI_PANEL_TIMER,
                              ATTR_ENABLED, 0);
            quitPressed = 1;
            QuitUserInterface (0);
            break;
    }
    return 0;
}

static void AIDisplayError (short error)
{
    static int lastError = 0;
    static int totalLines;

    if (error != lastError) 
    {   
        if (error ==  overWriteErr)
            displayOverWriteOccurred = 1;
    
        if (displayOverWriteOccurred)
        {
            SetCtrlAttribute (AIpanelHandle, AI_PANEL_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_RED);
            ResetTextBox (AIpanelHandle, AI_PANEL_ERRORBOX, 
                        "Overwrite error has occurred!! Press Clear.\n(Try setting Read Mode to \"Latest\".)");
            lastError = overWriteErr;
            return;
        }

        if (error || etsError || analogTrigError)
        {
            if (etsError)
            {
                error = etsError;
            }
            else if (analogTrigError)
            {
                error = analogTrigError;
            }
            
            if (error != lastError)
            {
                SetCtrlAttribute (AIpanelHandle, AI_PANEL_ERRORBOX, ATTR_TEXT_COLOR,
                                  VAL_RED);
                ResetTextBox (AIpanelHandle, AI_PANEL_ERRORBOX, 
                            GetDAQErrorString(error));
            
                SetCtrlVal (AIpanelHandle, AI_PANEL_ERRORBOX, "\n");
        
                if (error == timeOutErr || error == -timeOutErr)
                {
                    GetCtrlAttribute (AIpanelHandle, AI_PANEL_ERRORBOX, ATTR_TOTAL_LINES,
                                      &totalLines);
        
                    InsertTextBoxLine (AIpanelHandle, AI_PANEL_ERRORBOX, totalLines-1, "Operations will be slow while the timeout exists!  "
                                                                                       "Make sure your trigger specification is correct for your signal.");
                }
            }
        }
        else
        {
            SetCtrlAttribute (AIpanelHandle, AI_PANEL_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_BLACK);
            ResetTextBox (AIpanelHandle, AI_PANEL_ERRORBOX, 
                        GetDAQErrorString(error));
        }

        lastError = error;
    }
}

