/************************************************************************/
/*                                                                      */
/*  This module shows you how to log data to disk while asynchronously  */
/*  acquiring data.  Data can be stored in ASCII format for importing   */
/*  into a spreadsheet or stored as binary to optimize for speed.       */                                                   
/*                                                                      */
/*  This program also incorporates the functionality of ai_async.c      */
/*  which performs an asynchronous timed analog                         */
/*  acquisition using AIStartAcquisition(), AICheckAcquisition(),       */
/*  AIReadAcquisition(), and AIClearAcquisition().                      */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <utility.h>
#include <analysis.h>
#include <ansi_c.h>
#include <userint.h>
#include <formatio.h>
#include "easyio.h"
#include "toolbox.h"
#include "ai_async.h"
#include "ai_log.h"

#define NUM_UPDATES 100

/****  globals shared with ai_async.c ****/

extern int  gAIpanelHandle;
extern int  gAIlogging;
extern int ai_main (int argc, char *argv[]);

/*****************************************/

static int AILogConfigHandle;
static char projectPath[MAX_PATHNAME_LEN];

static struct logAttributes
{
    char filename[MAX_PATHNAME_LEN];
    int  fileAction;
    int  fileType;
    int  columnSepStyle;
    int  fieldWidth;
} dataLogAttributes;


static void GetLogFileAttributes(void);
static void SetLogFileAttributes(void);


int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    ai_main(argc, argv);
    
    AILogConfigHandle = LoadPanel (0, "ai_async.uir", LOGCONFIG);
    
    SetCtrlVal (gAIpanelHandle, AI_PANEL_SCANRATE, 1000.0);
    AIScanRateCallback (gAIpanelHandle, AI_PANEL_SCANRATE, EVENT_VAL_CHANGED,
        0, 0, 0);

    SetCtrlVal (gAIpanelHandle, AI_PANEL_NUMSCANS, 200);
    AINumScansCallback (gAIpanelHandle, AI_PANEL_NUMSCANS, EVENT_VAL_CHANGED,
        0, 0, 0);
    
    SetCtrlAttribute (gAIpanelHandle, AI_PANEL_LOGBUTTON, ATTR_VISIBLE, 1);
    SetCtrlAttribute (gAIpanelHandle, AI_PANEL_LOGCONFIG, ATTR_VISIBLE, 1);
    SetCtrlAttribute (gAIpanelHandle, AI_PANEL_CLEAR, ATTR_VISIBLE, 0);
    SetPanelAttribute (gAIpanelHandle, ATTR_TITLE,
                       "Asynchronous Acquisition with Data Logging");
    
    GetProjectDir (projectPath);
    sprintf (dataLogAttributes.filename, "%s\\data.txt", 
                                projectPath);
    SetCtrlVal (AILogConfigHandle, LOGCONFIG_FILENAME, dataLogAttributes.filename);
    GetLogFileAttributes ();
    DisplayPanel (gAIpanelHandle);

    RunUserInterface ();

    return 0;
}

int CVICALLBACK LogConfigCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        static int textLength;
        case EVENT_COMMIT:
            GetLogFileAttributes ();
            InstallPopup (AILogConfigHandle);
            break;
    }
    return 0;
}

int CVICALLBACK LogPopupCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int  int_dummy;
    static char string_dummy[MAX_PATHNAME_LEN];
    
    switch (event) {
        case EVENT_COMMIT:
            switch (control) {
                case LOGCONFIG_CANCEL:
                    RemovePopup (0);
                    SetLogFileAttributes ();
                break;
                case LOGCONFIG_OK:
                    RemovePopup (0);
                    GetLogFileAttributes ();
                break;
                case LOGCONFIG_FILETYPE:
                    GetCtrlVal (AILogConfigHandle, LOGCONFIG_FILETYPE, &int_dummy);
                    if (int_dummy == VAL_BINARY)
                    {
                        SetCtrlAttribute (AILogConfigHandle, LOGCONFIG_SEPSTYLE, ATTR_DIMMED, 1);
                        SetCtrlAttribute (AILogConfigHandle, LOGCONFIG_WIDTH, ATTR_DIMMED, 1);
                        ChangeFileNameExtension(dataLogAttributes.filename, string_dummy, "bin");   /* no function panel yet */
                    
                    }
                    else
                    {
                        SetCtrlAttribute (AILogConfigHandle, LOGCONFIG_SEPSTYLE, ATTR_DIMMED, 0);
                        SetCtrlAttribute (AILogConfigHandle, LOGCONFIG_WIDTH, ATTR_DIMMED, 0);
                        ChangeFileNameExtension(dataLogAttributes.filename, string_dummy, "txt");   /* no function panel yet */
                    }
                    SetCtrlVal (AILogConfigHandle, LOGCONFIG_FILENAME, string_dummy);
                break;
                case LOGCONFIG_SEPSTYLE:
                    GetCtrlVal (AILogConfigHandle, LOGCONFIG_SEPSTYLE, &int_dummy);
                    if (int_dummy == VAL_CONST_WIDTH)
                    {
                        SetCtrlAttribute (AILogConfigHandle, LOGCONFIG_WIDTH, ATTR_DIMMED, 0);
                    }
                    else
                    {
                        SetCtrlAttribute (AILogConfigHandle, LOGCONFIG_WIDTH, ATTR_DIMMED, 1);
                    }
                break;
                case LOGCONFIG_BROWSE:
                    GetCtrlVal (AILogConfigHandle, LOGCONFIG_FILETYPE, &int_dummy);
                    if (int_dummy == VAL_BINARY)
                    {
                        FileSelectPopup (projectPath, "*.bin", "", "Select Log File",
                                     VAL_LOAD_BUTTON, 0, 0, 1, 1,
                                     dataLogAttributes.filename);
                    }
                    else
                    {
                        FileSelectPopup (projectPath, "*.txt", "", "Select Log File",
                                     VAL_LOAD_BUTTON, 0, 0, 1, 1,
                                     dataLogAttributes.filename);
                    }
                    SetCtrlVal (gAIpanelHandle, LOGCONFIG_FILENAME, dataLogAttributes.filename);
                break;
            
            }
            
            break;
    }
    return 0;
}

static void GetLogFileAttributes()
{
    GetCtrlVal (AILogConfigHandle, LOGCONFIG_FILENAME,
                dataLogAttributes.filename);
    GetCtrlVal (AILogConfigHandle, LOGCONFIG_ACTION,
                &dataLogAttributes.fileAction);
    GetCtrlVal (AILogConfigHandle, LOGCONFIG_FILETYPE,
                &dataLogAttributes.fileType);
    GetCtrlVal (AILogConfigHandle, LOGCONFIG_SEPSTYLE,
                &dataLogAttributes.columnSepStyle);
    GetCtrlVal (AILogConfigHandle, LOGCONFIG_WIDTH,
                &dataLogAttributes.fieldWidth);


}

static void SetLogFileAttributes()
{
    SetCtrlVal (AILogConfigHandle, LOGCONFIG_FILENAME,
                dataLogAttributes.filename);
    SetCtrlVal (AILogConfigHandle, LOGCONFIG_ACTION,
                dataLogAttributes.fileAction);
    SetCtrlVal (AILogConfigHandle, LOGCONFIG_FILETYPE,
                dataLogAttributes.fileType);
    SetCtrlVal (AILogConfigHandle, LOGCONFIG_SEPSTYLE,
                dataLogAttributes.columnSepStyle);
    SetCtrlVal (AILogConfigHandle, LOGCONFIG_WIDTH,
                dataLogAttributes.fieldWidth);


}


int CVICALLBACK LogCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (gAIpanelHandle, AI_PANEL_LOGBUTTON, &gAIlogging);
            break;
    }
    return 0;
}


int LogWaveform (double *gAIwaveforms, int totalPoints, int numChans)
{
    int error;
    
    error = ArrayToFile (dataLogAttributes.filename, gAIwaveforms, VAL_DOUBLE,
                             totalPoints, numChans,
                             VAL_GROUPS_TOGETHER, VAL_GROUPS_AS_COLUMNS,
                             dataLogAttributes.columnSepStyle, 
                             dataLogAttributes.fieldWidth, dataLogAttributes.fileType, 
                             dataLogAttributes.fileAction);

    return error;
}
