/************************************************************************/
/*                                                                      */
/*  This program shows you how to read analog input channels using      */
/*  AISampleChannels() and plot the results to a strip chart.           */
/*  The acquisition and plotting is performed in TimerCallback().       */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <utility.h>
#include <ansi_c.h>
#include "easyio.h"
#include "ai_samp.h"

void ErrorHandler (short error);
                                
static int            panelHandle;
static int            helpHandle;
static short          device;
static unsigned long  numChannels = 0;
static char           channelString[100];
static double          upper;
static double          lower;
static double          rate;
static double          voltages[100];

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    panelHandle = LoadPanel (0, "ai_samp.uir", PANEL);
    helpHandle = LoadPanel (0, "ai_samp.uir", HELP_PANEL);
    DeviceCallback (panelHandle, PANEL_DEVICE, EVENT_COMMIT,
        0, 0, 0);
    ChannelCallback (panelHandle, PANEL_CHANNEL, EVENT_COMMIT,
        0, 0, 0);
    LimitCallback (panelHandle, PANEL_UPPER, EVENT_COMMIT,
        0, 0, 0);
    LimitCallback (panelHandle, PANEL_LOWER, EVENT_COMMIT,
        0, 0, 0);
    RateCallback (panelHandle, PANEL_RATE, EVENT_COMMIT,
        0, 0, 0);
    DisplayPanel (panelHandle);
    RunUserInterface ();

    return 0;
}

int CVICALLBACK DeviceCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle, PANEL_DEVICE, &device);
            ChannelCallback (panelHandle, PANEL_CHANNEL, EVENT_COMMIT,
                0, 0, 0);
            break;
    }
    return 0;
}

int CVICALLBACK ChannelCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int error;
    
    switch (event) {
        case EVENT_RIGHT_CLICK:
            DisplayPanel(helpHandle);
            break;
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle, PANEL_CHANNEL, channelString);
            
            DisableBreakOnLibraryErrors ();
            
            error = GetNumChannels (device, channelString, ANALOG_INPUT, 
                                &numChannels);
            
            EnableBreakOnLibraryErrors ();
            
            ErrorHandler (error);
            
            if (numChannels)
                SetCtrlAttribute (panelHandle, PANEL_STRIPCHART, 
                                ATTR_NUM_TRACES, numChannels);
            
            if (numChannels > 1)
            {
                SetTraceAttribute (panelHandle, PANEL_STRIPCHART, 2,
                               ATTR_TRACE_COLOR, VAL_YELLOW);
            }
            if (numChannels > 2)
            {
                SetTraceAttribute (panelHandle, PANEL_STRIPCHART, 3,
                               ATTR_TRACE_COLOR, VAL_CYAN);
            }
            break;
    }
    return 0;
}

int CVICALLBACK ChannelRingCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    
    char  * str;
    switch (event) {
        static int index;
        static int length;
        case EVENT_COMMIT:
            
            GetCtrlIndex (panelHandle, PANEL_CHANNELRING, &index);
            GetValueLengthFromIndex (panelHandle, PANEL_CHANNELRING, index,
                                     &length);
            str = (char *) malloc (length+1);
            GetCtrlVal (panelHandle, PANEL_CHANNELRING, str);
            SetCtrlVal (panelHandle, PANEL_CHANNEL, str);
            ChannelCallback (panelHandle, PANEL_CHANNEL, EVENT_COMMIT,
                0, 0, 0);
            free (str);
            break;
    }
    return 0;
}



int CVICALLBACK LimitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            switch (control)
            {
                case PANEL_UPPER:
                    GetCtrlVal (panelHandle, PANEL_UPPER, &upper);
                    if (upper <= lower)
                    {
                        upper = lower + 1.0;
                        SetCtrlVal (panelHandle, PANEL_UPPER, upper);
                    
                    }
                break;
                case PANEL_LOWER:
                    GetCtrlVal (panelHandle, PANEL_LOWER, &lower);
                    if (upper <= lower)
                    {
                        lower = upper - 1.0;
                        SetCtrlVal (panelHandle, PANEL_LOWER, lower);
                    }
                
                break;
            }
            SetAxisRange (panelHandle, PANEL_STRIPCHART, VAL_NO_CHANGE,
                              0.0, 1.0, VAL_MANUAL, lower, upper);
            break;
    }
    return 0;
}


int CVICALLBACK RateCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)

{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle, PANEL_RATE, &rate);
            SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_INTERVAL,
                              1/rate);
            break;
    }
    return 0;
}

int CVICALLBACK TimerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    short error;
    static int errorMsgActive = 0;
    static int inc = 0;
    static int needToRestartActualRateCalc = 1;
    static double startTime, elapsedTime;
    
    switch (event) {
        case EVENT_TIMER_TICK:
            DisableBreakOnLibraryErrors ();
            
            error = AISampleChannels (device, channelString, upper, lower, 
                                voltages);
            EnableBreakOnLibraryErrors ();

            ErrorHandler (error);
            
            if (numChannels)
                PlotStripChart (panelHandle, PANEL_STRIPCHART, voltages,
                            numChannels, 0, 0, VAL_DOUBLE);
            
            
            if (needToRestartActualRateCalc)
            {
                startTime = Timer();
                needToRestartActualRateCalc = 0;
                inc = 0;
            }

            if ((elapsedTime = (Timer() - startTime)) >= 1.0)
            {
                SetCtrlVal (panelHandle, PANEL_ACTUAL_RATE, inc/elapsedTime);
                needToRestartActualRateCalc = 1;
            }

            inc++;
            
            break;
    }
    return 0;
}

int CVICALLBACK CloseHelpCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            HidePanel (panel);
            break;
    }
    return 0;
}

int CVICALLBACK QuitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_ENABLED, 0);
            QuitUserInterface (0);
            break;
    }
    return 0;
}


void ErrorHandler (short error)
{
    static int lastError = 0;

    
    if (error != lastError)
    {
        
        if (error == 0)
        {
            SetCtrlAttribute (panelHandle, PANEL_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_BLACK);
        }
        else
        {
            SetCtrlAttribute (panelHandle, PANEL_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_RED);
        
        }
        
        ResetTextBox (panelHandle, PANEL_ERRORBOX, 
                        GetDAQErrorString(error));
    }
    lastError = error;
}

