/************************************************************************/
/*                                                                      */
/*  This program shows you how to perform an asynchronous timed analog  */
/*  waveform generation using AOGenerateWaveforms() and                 */
/*  AOClearWaveforms().                                                 */
/*  A waveform editor is provided on the panel for customizing the      */
/*  waveform.                                                           */
/*                                                                      */
/*  This program also incorporates the functionality of ai_async.prj.   */
/*  You can tie the analog output channel to the analog input channel   */
/*  in order to acquire and view the waveform that is generated.        */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <utility.h>
#include <analysis.h>
#include <ansi_c.h>
#include <userint.h>
#include "ao_wave.h"
#include "ai_async.h"
#include "easyio.h"

#define NUM_UPDATES 100

/****  globals shared with ai_async.c ****/

extern int           gAIpanelHandle;
extern int ai_main (int argc, char *argv[]);

/*****************************************/

void AODisplayError (short error);

int Interpolate (double previousX, double previousY, 
                    double cursorX, double cursorY);

static int    asyncRunning = 0;
static int    AOpanelHandle;
static int    AOhelpHandle;
static unsigned long AOtaskID;
static short  AOdevice;
static unsigned long  AOnumChannels;
static char   AOchannelString[100];
static double  AOupdateRate;
static double  AOGraphData[NUM_UPDATES + 1] = {0.0};
static double  * AOwaveforms = 0;


static int ColorArray[8] = {VAL_GREEN, VAL_YELLOW, VAL_CYAN, VAL_WHITE, VAL_BLUE, 
                            VAL_MAGENTA, VAL_GRAY, VAL_RED};

int main (int argc, char *argv[])
{
    int i;
    
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    AOpanelHandle = LoadPanel (0, "ao_wave.uir", AO_PANEL);
    AOhelpHandle = LoadPanel (0, "ao_wave.uir", AO_HELP);
    
    GetCtrlVal (AOpanelHandle, AO_PANEL_DEVICE, &AOdevice);
    GetCtrlVal (AOpanelHandle, AO_PANEL_CHANNEL, AOchannelString);
    GetCtrlVal (AOpanelHandle, AO_PANEL_UPDATERATE, &AOupdateRate);
    
    for (i=0; i<NUM_UPDATES; i++)
    {
        AOGraphData[i] = ((double)i/NUM_UPDATES * 5.0 - 2.5);
    }

    AOGraphCallback (AOpanelHandle, 0, EVENT_VAL_CHANGED, 0, 0, 0);  
    AOGraphCallback (AOpanelHandle, 0, EVENT_COMMIT, 0, 0, 0);  
    SetPanelAttribute (AOpanelHandle, ATTR_CLOSE_CTRL, AO_PANEL_QUIT);
    SetAxisRange (AOpanelHandle, AO_PANEL_GRAPH, VAL_MANUAL, 0.0, 
                (double)(NUM_UPDATES-1), VAL_NO_CHANGE, 0.0, 1.0);
    ai_main(argc, argv);
    
    

    SetCtrlVal (gAIpanelHandle, AI_PANEL_UPPER, 5.0);
    AILimitCallback (gAIpanelHandle, AI_PANEL_UPPER, EVENT_VAL_CHANGED,
        0, 0, 0);
    SetCtrlVal (gAIpanelHandle, AI_PANEL_LOWER, -5.0);
    AILimitCallback (gAIpanelHandle, AI_PANEL_LOWER, EVENT_VAL_CHANGED,
        0, 0, 0);
    SetCtrlVal (gAIpanelHandle, AI_PANEL_SCANRATE, 602.0);
    AIScanRateCallback (gAIpanelHandle, AI_PANEL_SCANRATE, EVENT_VAL_CHANGED,
        0, 0, 0);
    SetCtrlVal (gAIpanelHandle, AI_PANEL_NUMSCANS, NUM_UPDATES);
    AINumScansCallback (gAIpanelHandle, AI_PANEL_SCANRATE, EVENT_VAL_CHANGED,
        0, 0, 0);
    SetCtrlVal (gAIpanelHandle, AI_PANEL_READMODE,
                CONSECUTIVE_MODE);
    AIReadModeCallback (gAIpanelHandle, AI_PANEL_READMODE, EVENT_COMMIT,
        0, 0, 0);
    
    SetCtrlAttribute (gAIpanelHandle, AI_PANEL_QUIT, ATTR_VISIBLE,
                      0);
    
    SetCtrlAttribute (gAIpanelHandle, AI_PANEL_CLEAR, ATTR_VISIBLE, 0);
    
    
    SetPanelPos (gAIpanelHandle, 225, VAL_AUTO_CENTER);

    SetPanelAttribute (AOpanelHandle, ATTR_CLOSE_CTRL, AO_PANEL_QUIT);
    
    DisplayPanel (gAIpanelHandle);

    DisplayPanel (AOpanelHandle);

    RunUserInterface ();

    return 0;
}


int CVICALLBACK AODeviceCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
     int error;
     
     switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (AOpanelHandle, AO_PANEL_DEVICE, &AOdevice);
            AOChannelCallback (AOpanelHandle, AO_PANEL_CHANNEL, EVENT_COMMIT,
                0, 0, 0);
            break;
    }
    return 0;
}


int CVICALLBACK AOChannelCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    
    int i, j, k;
    int error;
    
    switch (event) {
        case EVENT_RIGHT_CLICK:
            DisplayPanel(AOhelpHandle);
            break;
        case EVENT_COMMIT:
            GetCtrlVal (AOpanelHandle, AO_PANEL_CHANNEL, AOchannelString);
            DisableBreakOnLibraryErrors ();
            if (asyncRunning)
            {
                AOClearWaveforms (AOtaskID);
                asyncRunning = 0;
            }
            error = GetNumChannels (AOdevice, AOchannelString, ANALOG_OUTPUT, 
                                &AOnumChannels);
            EnableBreakOnLibraryErrors ();
            AODisplayError (error);
            if (error)
                return (0);
            
            free (AOwaveforms);
            AOwaveforms = malloc (NUM_UPDATES * AOnumChannels * 
                                    sizeof (double));
            
            /*********  Multiplex the data  *********/
            k = 0;
            for (i=0; i<NUM_UPDATES; i++)
            {
                for (j=0; j<AOnumChannels; j++)
                {
                    AOwaveforms[k++] = AOGraphData[i];
                }   
            }
            /****************************************/
            
            DisableBreakOnLibraryErrors ();
            error = AOGenerateWaveforms (AOdevice, AOchannelString,
                                   AOupdateRate, NUM_UPDATES, 0,
                                   AOwaveforms, &AOtaskID);
            EnableBreakOnLibraryErrors ();
            AODisplayError (error);
            if (error >= 0)
                asyncRunning = 1;
            break;
    }
    return 0;
}

int CVICALLBACK AOChannelRingCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    
    char  * str;
    switch (event) {
        static int index;
        static int length;
        case EVENT_COMMIT:
            
            GetCtrlIndex (AOpanelHandle, AO_PANEL_CHANNELRING, &index);
            GetValueLengthFromIndex (AOpanelHandle, AO_PANEL_CHANNELRING, index,
                                     &length);
            str = (char *) malloc (length+1);
            GetCtrlVal (AOpanelHandle, AO_PANEL_CHANNELRING, str);
            SetCtrlVal (AOpanelHandle, AO_PANEL_CHANNEL, str);
            AOChannelCallback (AOpanelHandle, AO_PANEL_CHANNEL, EVENT_COMMIT,
                0, 0, 0);
            free (str);
            str = 0;
            break;
    }
    return 0;
}


int CVICALLBACK AOUpdateRateCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)

{
    int error;
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (AOpanelHandle, AO_PANEL_UPDATERATE, &AOupdateRate);
            DisableBreakOnLibraryErrors ();
            if (asyncRunning)
            {
                AOClearWaveforms (AOtaskID);
                asyncRunning = 0;
            }
            error = AOGenerateWaveforms (AOdevice, AOchannelString,
                                   AOupdateRate, NUM_UPDATES, 0,
                                   AOwaveforms, &AOtaskID);
            EnableBreakOnLibraryErrors ();
            AODisplayError (error);
            if (error >= 0)
                asyncRunning = 1;
            break;
    }
    return 0;
}



int CVICALLBACK AOCloseHelpCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            HidePanel (AOhelpHandle);
            break;
    }
    return 0;
}

int CVICALLBACK AOGraphCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    static double previousX;
    static double previousY;
    static double cursorX;
    static double cursorY;
    static int mouseDown = 0;
    
    int x;
    
    switch (event) {
        case EVENT_VAL_CHANGED:
            GetGraphCursor (AOpanelHandle, AO_PANEL_GRAPH, 1, &cursorX,
                            &cursorY);
            
            if (cursorX == 0)
                cursorX = 0.1;
                
            if (mouseDown && (fabs(cursorX - previousX) > 1.0))
            {
                Interpolate (previousX, previousY, cursorX, cursorY);
            
            }
            else
            {
                if ((int) cursorX == 0)
                {
                    AOGraphData[(int) cursorX] = (double) cursorY;
                }
            
                x = (int)ceil(cursorX);

                AOGraphData[x] = (double) cursorY;

            }
            
            previousX = cursorX;
            previousY = cursorY; 
            mouseDown = 1;
            
            DeleteGraphPlot (AOpanelHandle, AO_PANEL_GRAPH, -1,
                             VAL_DELAYED_DRAW);
            PlotY (AOpanelHandle, AO_PANEL_GRAPH, AOGraphData, NUM_UPDATES,
                   VAL_DOUBLE, VAL_FAT_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1,
                   VAL_GREEN);

            break;

        
        case EVENT_COMMIT:

            AOChannelCallback(AOpanelHandle, 0, EVENT_COMMIT, 0, 0, 0);
            
            mouseDown = 0;
            break;
    }
    return 0;
}


int CVICALLBACK AOQuitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            QuitCallback (0, 0, EVENT_COMMIT, 0, 0, 0);         
            DisableBreakOnLibraryErrors ();
            AOClearWaveforms (AOtaskID);
            EnableBreakOnLibraryErrors ();
            QuitUserInterface (0);
            break;
    }
    return 0;
}


void AODisplayError (short error)
{
    static int lastError = 0;

    
    if (error != lastError)
    {
        
        if (error == 0)
        {
            SetCtrlAttribute (AOpanelHandle, AO_PANEL_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_BLACK);
        }
        else
        {
            SetCtrlAttribute (AOpanelHandle, AO_PANEL_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_RED);
        
        }
        
        ResetTextBox (AOpanelHandle, AO_PANEL_ERRORBOX, 
                        GetDAQErrorString(error));
    }
    lastError = error;
}



int Interpolate (double previousX, double previousY, 
                    double currentX, double currentY)
{
    double i;
    int k;
    double slope;
    double yInit;
    double x1, x2;
    int temp;
    
    slope = (currentY - previousY)/(currentX - previousX);
    if (slope > 0)
    {
        
        yInit = (currentY > previousY)? previousY: currentY;
    }
    else
    {
        yInit = (currentY > previousY)? currentY: previousY;
    }
    if (currentX > previousX)
    {
        x1 = previousX;
        x2 = currentX;
    }
    else
    {
        x1 = currentX;
        x2 = previousX;
    }
    
    k = 0;
    for (i=x1; (int)i<=(int)x2; i++)
    {
        if ((int)i == 0)
        {
            AOGraphData[(int) i] = (double) (yInit + (k * slope));
        }
            
        temp = (int)ceil(i);
        
        AOGraphData[temp] = (double) (yInit + (k * slope));

        if (AOGraphData[temp] > 10.0)
        {
            AOGraphData[temp] = 10.0;   
        }
        if (AOGraphData[temp] < -10.0)
        {
            AOGraphData[temp] = -10.0;   
        }
        k++;
    }
    
    return (0);
}

