/************************************************************************/
/*                                                                      */
/*  This program shows you how to use a DAQ-STC or Am9513 counter to    */
/*  count TTL edges of the signal on the specified counter's SOURCE pin */
/*  or the number of cycles of a specified internal timebase signal.    */
/*                                                                      */
/*  When the internal timebase is used, CounterEventOrTimeConfig() in   */
/*  conjunction with CounterStart() and CounterRead() can be used to    */
/*  make more precise timing measurements than with the Timer()         */
/*  function.                                                           */
/*                                                                      */
/*  If you select "Use Counter Source" from the Source/Timebase         */
/*  control, you must supply a TTL signal to the SOURCE pin of this     */
/*  counter.                                                            */
/*                                                                      */
/*  This project also incorporates the functionality of ctr_puls.c and  */
/*  ctr_puls.uir to generate pulses to count.  Tie the OUT pin of the   */
/*  counter generating pulses to the SOURCE pin of the counter that is  */
/*  counting pulses to monitor the output.                              */
/*                                                                      */
/************************************************************************/


#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <ansi_c.h>
#include <userint.h>
#include <utility.h>
#include "easyio.h"
#include "ctr_evt.h"

static int evtCount;
static int daqError = 0;
static unsigned long taskID = 0;
static int BOLEstate;

static int CtrDisplayError (short error);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    evtCount = LoadPanel (0, "ctr_evt.uir", EVT_COUNT);
    DisplayPanel (evtCount);
    SetCtrlAttribute (evtCount, EVT_COUNT_TIMER, ATTR_ENABLED, 0);
    RunUserInterface ();

    return 0;
}


int CVICALLBACK StartCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    short device;
    int intCounter;
    double source;
    unsigned short counterSize;
    char strCounter[10];
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (evtCount, EVT_COUNT_DEVICE, &device);
            GetCtrlVal (evtCount, EVT_COUNT_COUNTER, &intCounter);
            GetCtrlVal (evtCount, EVT_COUNT_SIZE, &counterSize);
            GetCtrlVal (evtCount, EVT_COUNT_SOURCE, &source);
            sprintf (strCounter, "%d", intCounter);
            
            if (taskID)
            {
                BOLEstate = GetBreakOnLibraryErrors ();
                if (BOLEstate)
                    DisableBreakOnLibraryErrors ();
                daqError = CounterStop (taskID);
                if (BOLEstate)
                    EnableBreakOnLibraryErrors ();
                CtrDisplayError (daqError);
            }
            
            BOLEstate = GetBreakOnLibraryErrors ();
            if (BOLEstate)
                DisableBreakOnLibraryErrors ();


            daqError = CounterEventOrTimeConfig (device, strCounter, counterSize,
                                      source,
                                      COUNT_CONTINUOUSLY,
                                      COUNT_ON_RISING_EDGE,
                                      UNGATED_SOFTWARE_START, &taskID);
            
            if (BOLEstate)
                EnableBreakOnLibraryErrors ();

            
            if (CtrDisplayError (daqError))
                return daqError;
                
            
            daqError = CounterStart (taskID);
            
            if (CtrDisplayError (daqError))
                return daqError;

            
            SetInputMode (evtCount, EVT_COUNT_DEVICE, 0);
            SetInputMode (evtCount, EVT_COUNT_SOURCE, 0);
            SetInputMode (evtCount, EVT_COUNT_SIZE, 0);
            SetInputMode (evtCount, EVT_COUNT_COUNTER, 0);
            
            
            SetCtrlAttribute (evtCount, EVT_COUNT_TIMER, ATTR_ENABLED, 1);
            
            break;
    }
    return 0;
}


int CVICALLBACK StopCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:

            if (taskID)
            {   
                BOLEstate = GetBreakOnLibraryErrors ();
                if (BOLEstate)
                    DisableBreakOnLibraryErrors ();
                daqError = CounterStop (taskID);
                if (BOLEstate)
                    EnableBreakOnLibraryErrors ();
                if (CtrDisplayError (daqError))
                    return daqError;
            }
            
            SetCtrlAttribute (evtCount, EVT_COUNT_TIMER, ATTR_ENABLED, 0);

            SetInputMode (evtCount, EVT_COUNT_DEVICE, 1);
            SetInputMode (evtCount, EVT_COUNT_SOURCE, 1);
            SetInputMode (evtCount, EVT_COUNT_SIZE, 1);
            SetInputMode (evtCount, EVT_COUNT_COUNTER, 1);
            
            break;
    }
    return 0;
}

int CVICALLBACK TimerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    short overflow;
    long count;
    
    switch (event) {
        case EVENT_TIMER_TICK:
            daqError = CounterRead (taskID, &count, &overflow);
            
            if (CtrDisplayError (daqError))
                return daqError;

            SetCtrlVal (evtCount, EVT_COUNT_READING, count);
            break;
    }
    return 0;
}

int CVICALLBACK QuitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            
            StopCallback (evtCount, EVT_COUNT_STOP, EVENT_COMMIT,
                0, 0, 0);
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}


static int CtrDisplayError (short error)
{
    static int lastError = 0;

    
    if (error != lastError)
    {

        if (error == 0)
        {
            SetCtrlAttribute (evtCount, EVT_COUNT_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_BLACK);
        }
        else
        {
            SetCtrlAttribute (evtCount, EVT_COUNT_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_RED);
        
        }
        ResetTextBox (evtCount, EVT_COUNT_ERRORBOX, 
                        GetDAQErrorString(error));
    }
    lastError = error;
    return error;
}

