/**********************************************************************/
/*                                                                    */
/*  This application is for DAQ devices that use DAQ-STC counters or  */
/*  Am9513 counters.                                                  */
/*                                                                    */
/*  This application measures the frequency of a TTL signal on the    */
/*  specified counter's SOURCE pin by counting rising edges of the    */
/*  signal during a specified period of time. In addition to this     */
/*  connection, you must wire the counter's GATE pin to counter-1's   */
/*  OUT pin.  Identify counter-1 from the table below.                */
/*                                                                    */
/*  This application is useful for relatively high frequency signals  */
/*  when many cycles of the signal occur during the timing period     */
/*  (gate width). Use ctr_puls.prj for relatively low frequency       */
/*  signals. Keep in mind that period(S) = 1/frequency (Hz).          */
/*                                                                    */
/*  On a counter overflow, use a larger counter size or decrease the  */
/*  gate width; if the number of rising edges is low, increase the    */
/*  gate width.                                                       */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/*                   Am9513                                           */
/*        --------------------------------                            */
/*        counter-1    |   counter                                    */
/*        --------------------------------                            */
/*                 5       |      1                                   */
/*                 1       |      2                                   */
/*                 2       |      3                                   */
/*                 3       |      4                                   */
/*                 4       |      5                                   */
/*                10       |      6                                   */
/*                 6       |      7                                   */
/*                 7       |      8                                   */
/*                 8       |      9                                   */
/*                 9       |     10                                   */
/*                                                                    */
/*                                                                    */
/*                  DAQ-STC                                           */
/*        --------------------------------                            */
/*        counter-1    |   counter                                    */
/*        ----------------------------------                          */
/*                 1       |      0                                   */
/*                 0       |      1                                   */
/*                                                                    */
/*                                                                    */
/*  Not all counters are available for all boards and operations;     */
/*  consult the DAQ VI and hardware reference manuals.                */
/*                                                                    */
/**********************************************************************/


#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <ansi_c.h>
#include <userint.h>
#include <utility.h>
#include "easyio.h"
#include "ctr_freq.h"

static int freqMeas;
static int daqError = 0;
static unsigned long taskID = 0;

static short device;
static int intCounter;
static double gateWidth;
static unsigned short counterSize;
static char strCounter[10];
static int BOLEstate;


static int CtrDisplayError (short error);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    freqMeas = LoadPanel (0, "ctr_freq.uir", FREQ_PANEL);
    DisplayPanel (freqMeas);
    SetPanelAttribute (freqMeas, ATTR_CLOSE_CTRL, FREQ_PANEL_QUIT);
    ConfigCallback (0, 0, EVENT_COMMIT, 0, 0, 0);
    RunUserInterface ();

    return 0;
}


int CVICALLBACK ConfigCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (freqMeas, FREQ_PANEL_DEVICE, &device);
            GetCtrlVal (freqMeas, FREQ_PANEL_COUNTER, &intCounter);
            GetCtrlVal (freqMeas, FREQ_PANEL_SIZE, &counterSize);
            GetCtrlVal (freqMeas, FREQ_PANEL_GATE_WIDTH, &gateWidth);
            sprintf (strCounter, "%hd", intCounter);
            
            
            break;
    }
    return 0;
}



int CVICALLBACK TimerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    double actualGateWidth;
    short overflow;
    short valid;
    short timeout;
    double frequency;
    
    switch (event) {
        case EVENT_TIMER_TICK:
            
            BOLEstate = GetBreakOnLibraryErrors ();
            if (BOLEstate)
                DisableBreakOnLibraryErrors ();
                
            
            daqError = CounterMeasureFrequency (device, strCounter, counterSize,
                                                gateWidth, 1.0,
                                                UNGATED_SOFTWARE_START,
                                                &actualGateWidth, &overflow,
                                                &valid, &timeout, &frequency);

            if (BOLEstate)
                EnableBreakOnLibraryErrors ();
            
            
            if (CtrDisplayError (daqError))
                return daqError;

            SetCtrlVal (freqMeas, FREQ_PANEL_FREQUENCY, frequency);
            SetCtrlVal (freqMeas, FREQ_PANEL_OVERFLOW, overflow);
            SetCtrlVal (freqMeas, FREQ_PANEL_INVALID, !valid);
            SetCtrlVal (freqMeas, FREQ_PANEL_TIMEOUT, timeout);
            
            break;
    }
    return 0;
}

int CVICALLBACK QuitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            
            SetCtrlAttribute (freqMeas, FREQ_PANEL_TIMER, ATTR_ENABLED, 0);
            QuitUserInterface (0);
            break;
    }
    return 0;
}


static int CtrDisplayError (short error)
{
    static int lastError = 0;

    
    if (error != lastError)
    {

        if (error == 0)
        {
            SetCtrlAttribute (freqMeas, FREQ_PANEL_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_BLACK);
        }
        else
        {
            SetCtrlAttribute (freqMeas, FREQ_PANEL_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_RED);
        
        }
        ResetTextBox (freqMeas, FREQ_PANEL_ERRORBOX, 
                        GetDAQErrorString(error));
    }
    lastError = error;
    return error;
}

