/**********************************************************************/
/*                                                                    */
/*  This application configures a DAQ-STC or Am9513 counter to        */
/*  measure the pulse width or period of a TTL signal connected to    */
/*  its GATE pin.                                                     */
/*                                                                    */
/*  The measurement is done by counting the number of cycles of the   */
/*  specified timebase between the appropriate starting and ending    */
/*  events.  The higher the timebase frequency, the greater the       */
/*  measurement accuracy will be.                                     */                   
/*                                                                    */
/*  You must tie the signal that you want to measure to the GATE pin  */
/*  of this counter.                                                  */
/*                                                                    */
/*  This example incorporates the functionality of ctr_puls.c as a    */
/*  pulse generator so you can tie the OUT pin of the counter that is */
/*  generating pulses to the GATE of the counter performing pulse     */
/*  measurement.                                                      */
/*                                                                    */
/**********************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <utility.h>
#include <userint.h>
#include <ansi_c.h>
#include "easyio.h"
#include "ctr_perd.h"
#include "ctr_puls.h"


/****  globals shared with ctr_puls.c ****/
extern int ctr_puls_main (int argc, char *argv[]);
extern int gCtrPulse;

/*****************************************/

static int ctrPeriodPanel = 0;
static int daqError = 0;
static unsigned long taskID = 0;
static double timebase;
static int BOLEstate;
static int needToRestart = 1;

static int CtrPeriodDisplayError (short error);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    ctrPeriodPanel = LoadPanel (0, "ctr_perd.uir", CTR_PERIOD);
    SetPanelAttribute (ctrPeriodPanel, ATTR_CLOSE_CTRL, CTR_PERIOD_QUIT);
    SetPanelPos (ctrPeriodPanel, 225, VAL_AUTO_CENTER);
    
    ctr_puls_main (argc, argv);
    SetCtrlAttribute (gCtrPulse, CTR_PULS_QUIT, ATTR_VISIBLE, 0);
    
    MeasConfigCallback (0, 0, EVENT_COMMIT, 0, 0, 0);

    DisplayPanel (gCtrPulse);
    DisplayPanel (ctrPeriodPanel);
    
    RunUserInterface();

    return 0;
}


int CVICALLBACK MeasConfigCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    short device;
    int intCounter;
    char strCounter[10];
    unsigned short int measType;
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (ctrPeriodPanel, CTR_PERIOD_DEVICE, &device);
            GetCtrlVal (ctrPeriodPanel, CTR_PERIOD_COUNTER, &intCounter);
            sprintf (strCounter, "%d", intCounter);
            GetCtrlVal (ctrPeriodPanel, CTR_PERIOD_TIMEBASE, &timebase);
            GetCtrlVal (ctrPeriodPanel, CTR_PERIOD_TYPE, &measType);

            if (taskID)
            {
                daqError = CounterStop (taskID);
            
                if (CtrPeriodDisplayError (daqError))
                    return daqError;
            }
            
            BOLEstate = GetBreakOnLibraryErrors ();
            if (BOLEstate)
                DisableBreakOnLibraryErrors ();

            daqError = PulseWidthOrPeriodMeasConfig (device, 
                                                     strCounter,
                                                     measType,
                                                     timebase, 
                                                     &taskID);
            if (BOLEstate)
                EnableBreakOnLibraryErrors ();
            
            if (CtrPeriodDisplayError (daqError))
                return daqError;

            needToRestart = 1;
            
            break;
    }
    return 0;
}

int CVICALLBACK TimerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    short overflow;
    long count;
    double startTime;
    int counterStartError = 0;
    
    switch (event) {
        case EVENT_TIMER_TICK:
            
            BOLEstate = GetBreakOnLibraryErrors ();
        
            if (BOLEstate)
                DisableBreakOnLibraryErrors ();

            if (needToRestart)
            {
                startTime = Timer();
                do 
                {
                    daqError = CounterStart (taskID);
                }
                while (daqError == gateSignalErr && Timer() - startTime < 1.0);

                if (CtrPeriodDisplayError (daqError))
                {
                    if (BOLEstate)
                        EnableBreakOnLibraryErrors ();
                    return daqError;
                }
                needToRestart = 0;
            }

            daqError = CounterRead (taskID, &count, &overflow);
            
            if (BOLEstate)
                EnableBreakOnLibraryErrors ();
            
            if (CtrPeriodDisplayError (daqError))
                return daqError;
            
            if (count > 0)
            {   needToRestart = 1;
                SetCtrlVal (ctrPeriodPanel, CTR_PERIOD_READING, (count/timebase) * 1000.0);
            }
            break;
    }
    return 0;
}



int CVICALLBACK CtrPeriodQuitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            CtrQuitCallback (0, 0, EVENT_COMMIT, 0, 0, 0);
            
            if (taskID)
            {    
                daqError = CounterStop (taskID);
            
                CtrPeriodDisplayError (daqError);
                taskID = 0;
                SetCtrlAttribute (ctrPeriodPanel, CTR_PERIOD_TIMER, ATTR_ENABLED, 0);
            
            }
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}


static int CtrPeriodDisplayError (short error)
{
    static int lastError = 0;

    
    if (error != lastError)
    {

        if (error == 0)
        {
            SetCtrlAttribute (ctrPeriodPanel, CTR_PERIOD_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_BLACK);
        }
        else
        {
            SetCtrlAttribute (ctrPeriodPanel, CTR_PERIOD_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_RED);
        
        }
        ResetTextBox (ctrPeriodPanel, CTR_PERIOD_ERRORBOX, 
                        GetDAQErrorString(error));
    }
    lastError = error;
    return error;
}

