/********************************************************************/
/*                                                                  */
/*  This program configures a DAQ-STC or Am9513 counter to generate */
/*  a continuous TTL pulse train or a delayed TTL pulse on its OUT  */
/*  pin.                                                            */
/*                                                                  */
/*  The signal is created by decrementing the counter twice, first  */
/*  for the delay to the pulse (phase 1), then for the pulse itself */
/*  (phase 2).  The function selects the highest resolution         */
/*  timebase to achieve the desired characteristics.                */
/*                                                                  */
/*  This project also incorporates ai_async.c and ai_async.uir so   */
/*  that you can monitor the pulses on an analog input channel.     */
/*  Tie the OUT pin of the specified counter to the analog input    */
/*  channel specified on ai_async.uir to monitor the pulses.        */
/*                                                                  */
/********************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>
#include <utility.h>
#include "ctr_puls.h"
#include "easyio.h"
#include "ai_async.h"

#define CONTINUOUS 1

#ifdef CTR_PULS_PRJ
    /****  globals shared with ai_async.c ****/
    extern int ai_main (int argc, char *argv[]);
    extern int gAIpanelHandle;
    /*****************************************/
#endif

int gCtrPulse;

static int daqError = 0;
static unsigned long taskID = 0;
static int BOLEstate;

static int CtrDisplayError (short error);

#ifdef CTR_PERIOD_PRJ
int ctr_puls_main (int argc, char *argv[])
{
#else
int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
#endif

    gCtrPulse = LoadPanel (0, "ctr_puls.uir", CTR_PULS);
    SetPanelAttribute (gCtrPulse, ATTR_CLOSE_CTRL, CTR_PULS_QUIT);
    SetPanelPos (gCtrPulse, 18, VAL_AUTO_CENTER);
    
    ConfigCallback (0, 0, EVENT_COMMIT, 0, 0, 0);
    DutyCycleCallback (0, CTR_PULS_DUTYCYCLE, EVENT_VAL_CHANGED, 0, 0, 0);              

    #ifdef CTR_PULS_PRJ
        ai_main (argc, argv);
        SetPanelPos (gAIpanelHandle, 225, VAL_AUTO_CENTER);
        SetCtrlAttribute (gAIpanelHandle, AI_PANEL_QUIT, ATTR_VISIBLE, 0);
        DisplayPanel (gCtrPulse);
        DisplayPanel (gAIpanelHandle);
        RunUserInterface();
    #endif

    return 0;
}


int CVICALLBACK ConfigCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    short device;
    int intCounter;
    char strCounter[10];
    double frequency, dutyCycle, actualFrequency, actualDutyCycle;
    double pulseDelay, pulseWidth, actualDelay, actualPulseWidth;
    int pulseMode;
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (gCtrPulse, CTR_PULS_DEVICE, &device);
            GetCtrlVal (gCtrPulse, CTR_PULS_COUNTER, &intCounter);
            sprintf (strCounter, "%d", intCounter);
            GetCtrlVal (gCtrPulse, CTR_PULS_FREQUENCY, &frequency);
            GetCtrlVal (gCtrPulse, CTR_PULS_DUTYCYCLE, &dutyCycle);
            GetCtrlVal (gCtrPulse, CTR_PULS_PULSEMODE, &pulseMode);

            if (taskID)
            {
                daqError = CounterStop (taskID);
            
                if (CtrDisplayError (daqError))
                    return daqError;
            }
            
            if (pulseMode == CONTINUOUS)
            {
                BOLEstate = GetBreakOnLibraryErrors ();
                if (BOLEstate)
                    DisableBreakOnLibraryErrors ();

                daqError = ContinuousPulseGenConfig (device, strCounter, frequency, dutyCycle,
                                      UNGATED_SOFTWARE_START,
                                      POSITIVE_POLARITY, &actualFrequency, &actualDutyCycle,
                                      &taskID);
                if (BOLEstate)
                    EnableBreakOnLibraryErrors ();

                if (CtrDisplayError (daqError))
                    return daqError;

                SetInputMode (gCtrPulse, CTR_PULS_SINGLEPULSE, 0);
            }
            else
            {
                pulseDelay = (1/frequency) * (1 - dutyCycle);
                pulseWidth = (1/frequency) * (dutyCycle);
                
                BOLEstate = GetBreakOnLibraryErrors ();
                if (BOLEstate)
                    DisableBreakOnLibraryErrors ();

                daqError = DelayedPulseGenConfig (device, strCounter, pulseDelay, pulseWidth, USE_INTERNAL_TIMEBASE,
                                    UNGATED_SOFTWARE_START,
                                    POSITIVE_POLARITY, &actualDelay, 
                                    &actualPulseWidth, &taskID);
                
                if (BOLEstate)
                    EnableBreakOnLibraryErrors ();
                
                if (CtrDisplayError (daqError))
                    return daqError;
            
                SetInputMode (gCtrPulse, CTR_PULS_SINGLEPULSE, 1);
                
                return 0;
            }
            daqError = CounterStart (taskID);
            
            if (CtrDisplayError (daqError))
                return daqError;
            
            break;
    }
    return 0;
}

int CVICALLBACK DutyCycleCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    
    double dutyCycle, inverseDutyCycle, dummy;
    double x[5] = {0, 0.5, 0.5, 1, 1};
    double y[5] = {0, 0,   1,   1, 0};
    
    switch (event) {
        case EVENT_VAL_CHANGED:
            switch (control) {
                case CTR_PULS_DUTYCYCLE:
                    GetCtrlVal (gCtrPulse, CTR_PULS_DUTYCYCLE, &dutyCycle);
                break;
                case CTR_PULS_GRAPH:
                    GetGraphCursor (gCtrPulse, CTR_PULS_GRAPH, 1, &inverseDutyCycle,
                                    &dummy);
                    dutyCycle = 1 - inverseDutyCycle;
                    SetCtrlVal (gCtrPulse, CTR_PULS_DUTYCYCLE, dutyCycle);
                break;
            }
            x[1] = x[2] = 1-dutyCycle;
            
            DeleteGraphPlot (gCtrPulse, CTR_PULS_GRAPH, -1, VAL_DELAYED_DRAW);
            PlotXY (gCtrPulse, CTR_PULS_GRAPH, x, y, 5, VAL_DOUBLE,
                    VAL_DOUBLE, VAL_FAT_STEP, VAL_EMPTY_SQUARE, VAL_SOLID,
                    1, VAL_GREEN);
            ConfigCallback (0, 0, EVENT_COMMIT, 0, 0, 0);
            
            break;
    }
    return 0;
}

int CVICALLBACK SinglePulseCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            daqError = CounterStart (taskID);
            
            if (CtrDisplayError (daqError))
                return daqError;

            break;
    }
    return 0;
}


int CVICALLBACK CtrQuitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            #ifdef CTR_PULS_PRJ
                QuitCallback (0, 0, EVENT_COMMIT, 0, 0, 0);
            #endif
            
            if (taskID)
            {    
                daqError = CounterStop (taskID);
            
                CtrDisplayError (daqError);
                taskID = 0;
            }
            QuitUserInterface (0);
            break;
    }
    return 0;
}


static int CtrDisplayError (short error)
{
    static int lastError = 0;

    if (error != lastError)
    {
        if (error == 0)
        {
            SetCtrlAttribute (gCtrPulse, CTR_PULS_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_BLACK);
        }
        else
        {
            SetCtrlAttribute (gCtrPulse, CTR_PULS_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_RED);
        }
        ResetTextBox (gCtrPulse, CTR_PULS_ERRORBOX, 
                        GetDAQErrorString(error));
    }
    lastError = error;
    return error;
}

