/************************************************************************/
/*                                                                      */
/*  This module shows you how to use an 8253 counter on the Lab         */
/*  boards, SCXI-1200, DAQPad-1200, PC-LPM-16, or the DAQCard-700 to    */
/*  generate square waves and single pulses.                            */
/*                                                                      */
/*  There are some external connections that need to be made on your    */
/*  DAQ device for proper operation of this example.  The connections   */
/*  are detailed in the Instructions section of ctri_pls.uir.           */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>
#include <utility.h>
#include "easyio.h"
#include "ctri_pls.h"
#include "ai_async.h"

#define CONTINUOUS 1

/****  globals shared with ai_async.c ****/
    extern int ai_main (int argc, char *argv[]);
    extern int gAIpanelHandle;
/*****************************************/

int gCtrPulse;


static int           daqError = 0;
static unsigned long taskID = 0;

static int           CtrDisplayError (short error);
static int           previousBOLEstate;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    gCtrPulse = LoadPanel (0, "ctri_pls.uir", CTR_PULS);
    SetPanelAttribute (gCtrPulse, ATTR_CLOSE_CTRL, CTR_PULS_QUIT);
    SetPanelPos (gCtrPulse, 18, VAL_AUTO_CENTER);
    
    ConfigCallback (0, 0, EVENT_COMMIT, 0, 0, 0);

    ai_main (argc, argv);
    SetPanelPos (gAIpanelHandle, 225, VAL_AUTO_CENTER);
    SetCtrlAttribute (gAIpanelHandle, AI_PANEL_QUIT, ATTR_VISIBLE, 0);
    DisplayPanel (gCtrPulse);
    DisplayPanel (gAIpanelHandle);
    RunUserInterface();
    
    return 0;
}

/* ConfigCallback() handles events on most of the controls on ctri_pls.uir. */

int CVICALLBACK ConfigCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    short device;
    int intCounter;
    unsigned short ticks;
    double frequency, dutyCycle, actualFrequency, actualDutyCycle;
    double pulseDelay, pulseWidth, actualDelay, actualPulseWidth;
    int pulseMode;
    unsigned short dummy;
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (gCtrPulse, CTR_PULS_DEVICE, &device);
            GetCtrlVal (gCtrPulse, CTR_PULS_COUNTER, &intCounter);
            GetCtrlVal (gCtrPulse, CTR_PULS_TICKS, &ticks);
            GetCtrlVal (gCtrPulse, CTR_PULS_MODE, &pulseMode);

            
            if (pulseMode == CONTINUOUS)
            {
            
                
                previousBOLEstate = GetBreakOnLibraryErrors ();
                if (previousBOLEstate)
                    DisableBreakOnLibraryErrors();

                daqError = ICounterControl (device, intCounter, I_SQUARE_WAVE_RATE_GENERATOR, ticks * 2,
                                 I_BINARY, I_LOW_STATE, &dummy);
                
                if (previousBOLEstate)
                    EnableBreakOnLibraryErrors();
                
                if (CtrDisplayError (daqError))
                    return daqError;

                SetInputMode (gCtrPulse, CTR_PULS_SINGLEPULSE, 0);
                
            }
            
            else
            {
             
                
                daqError = ICounterControl (device, intCounter, I_RESET, ticks,
                                            I_BINARY, I_HIGH_STATE, &dummy);

                if (control == CTR_PULS_SINGLEPULSE)    /* In this case, the user selected "Single Pulse" */
                {
                    daqError = ICounterControl (device, intCounter, I_TOGGLE_ON_TC, ticks,
                                                I_BINARY, I_LOW_STATE, &dummy);
                
                    if (CtrDisplayError (daqError))
                        return daqError;
                }
                
                SetInputMode (gCtrPulse, CTR_PULS_SINGLEPULSE, 1);
                
                return 0;
            
            }
            
            break;
    }
    return 0;
}


int CVICALLBACK CtrQuitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            QuitCallback (0, 0, EVENT_COMMIT, 0, 0, 0);
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}


static int CtrDisplayError (short error)
{
    static int lastError = 0;

    
    if (error != lastError)
    {

        if (error == 0)
        {
            SetCtrlAttribute (gCtrPulse, CTR_PULS_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_BLACK);
        }
        else
        {
            SetCtrlAttribute (gCtrPulse, CTR_PULS_ERRORBOX, ATTR_TEXT_COLOR,
                              VAL_RED);
        
        }
        ResetTextBox (gCtrPulse, CTR_PULS_ERRORBOX, 
                        GetDAQErrorString(error));
    }
    lastError = error;
    return error;
}

