/************************************************************************/
/*                                                                      */
/*  This program shows you how to log data to disk while asynchronously */
/*  acquiring data.  Data can be stored in ASCII format for importing   */
/*  into a spreadsheet or stored as binary to optimize for speed.       */                                                   
/*                                                                      */
/*  This program is purposely made concise and does not check for       */
/*  errors.                                                             */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <utility.h>
#include <ansi_c.h>
#include <easyio.h>
#include <formatio.h>
#include "ai^log.h"

#define UPPER_LIMIT    5.0
#define LOWER_LIMIT   -5.0
#define NUM_SCANS      100
#define CIRC_BUF_SIZE  10*NUM_SCANS
#define DAQ_RATE       2000.0
#define MAX_CHANS      10
#define ON             1
#define OFF            0

static unsigned long  taskID;
static int            panelHandle, runMode = OFF;
static short          device;
static unsigned long  numChannels;
static char           channelString[100];
static double         waveform[MAX_CHANS*NUM_SCANS];
static int            asciiLoggingOn   = 0;
static int            binaryLoggingOn = 0;
static int            fileHandle;

static char binaryFileName[MAX_PATHNAME_LEN];
static char asciiFileName[MAX_PATHNAME_LEN];
static char projDir[MAX_PATHNAME_LEN];


int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    GetProjectDir (projDir);
    MakePathname (projDir, "data.txt", asciiFileName);
    
    MakePathname (projDir, "data.bin", binaryFileName);
    
    /*** Clear the data files  ***/
    fileHandle = OpenFile (asciiFileName, VAL_WRITE_ONLY, VAL_TRUNCATE,
                           VAL_ASCII);
    CloseFile (fileHandle);
    fileHandle = OpenFile (binaryFileName, VAL_WRITE_ONLY, VAL_TRUNCATE,
                           VAL_BINARY);
    CloseFile (fileHandle);
    /*****************************/
    
    panelHandle = LoadPanel (0, "ai^log.uir", PANEL);
    DisplayPanel (panelHandle);
    RunUserInterface ();

    return 0;
}


int CVICALLBACK StartStopCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int error;
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle, PANEL_START_STOP, &runMode);
            if (runMode == ON)
            {
                GetCtrlVal (panelHandle, PANEL_DEVICE, &device);
                GetCtrlVal (panelHandle, PANEL_CHANNEL, channelString);
        
                error = GetNumChannels (device, channelString, ANALOG_INPUT, 
                                    &numChannels);
        
                error = AIStartAcquisition (device, channelString, CIRC_BUF_SIZE, DAQ_RATE, UPPER_LIMIT,
                                            LOWER_LIMIT, 0, &taskID);
        
                SetCtrlAttribute (panelHandle, PANEL_DEVICE, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle, PANEL_CHANNEL, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_ENABLED, 1);
            }
            if (runMode == OFF)
            {
                error = AIClearAcquisition (taskID);
            
                SetCtrlAttribute (panelHandle, PANEL_DEVICE, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle, PANEL_CHANNEL, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_ENABLED, 0);
            }
            break;
    }
    return 0;
}


int CVICALLBACK TimerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    short error, i;
    
    switch (event) {
        static unsigned long scanBacklog;
        case EVENT_TIMER_TICK:
            error = AICheckAcquisition (taskID, &scanBacklog);
            if (scanBacklog >= NUM_SCANS)
            {
                error = AIReadAcquisition (taskID, NUM_SCANS, LATEST_MODE,
                                           &scanBacklog, GROUP_BY_CHANNEL, waveform);
            
                DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_DELAYED_DRAW);

                for (i=0; i<numChannels; i++)
                {

                    PlotY (panelHandle, PANEL_GRAPH, &waveform[i*NUM_SCANS], NUM_SCANS,
                           VAL_DOUBLE, VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1,
                           VAL_GREEN);
                
                }
            
                if (asciiLoggingOn)
                {
                    
                    ArrayToFile (asciiFileName, waveform, VAL_DOUBLE, NUM_SCANS * numChannels,
                                 numChannels, VAL_GROUPS_TOGETHER, VAL_GROUPS_AS_COLUMNS,
                                 VAL_CONST_WIDTH, 15, VAL_ASCII, VAL_APPEND);
                
                }
                if (binaryLoggingOn)
                {
                    
                    ArrayToFile (binaryFileName, waveform, VAL_DOUBLE, NUM_SCANS  * numChannels,
                                 numChannels, VAL_GROUPS_TOGETHER, VAL_GROUPS_AS_COLUMNS,
                                 VAL_CONST_WIDTH, 15, VAL_BINARY, VAL_APPEND);
                
                }
            }
            break;
    }
    return 0;
}

int CVICALLBACK LogCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:

            switch (control) {
                case PANEL_LOG_ASCII_BUTTON:
                    GetCtrlVal (panelHandle, PANEL_LOG_ASCII_BUTTON, &asciiLoggingOn);
                    break;
                case PANEL_LOG_BINARY_BUTTON:
                    GetCtrlVal (panelHandle, PANEL_LOG_BINARY_BUTTON, &binaryLoggingOn);
                    break;
            }
            break;
    }
    return 0;
}

int CVICALLBACK QuitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int error;
    switch (event) {
        case EVENT_COMMIT:
            if (runMode == ON)
                error = AIClearAcquisition (taskID);
            SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_ENABLED, 0);
            QuitUserInterface (0);
            break;
    }
    return 0;
}



