/************************************************************************/
/*                                                                      */
/*  This program shows you how to perform a triggered acquisition using */
/*  AIAcquireTriggeredWaveforms() and plot the results to a graph.      */
/*  The acquisition and plotting is performed in TimerCallback().       */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <utility.h>
#include <ansi_c.h>
#include <easyio.h>
#include "ai^trig.h"

#define UPPER_LIMIT    5.0
#define LOWER_LIMIT   -5.0
#define NUM_SCANS      100
#define MAX_CHANS      10

static unsigned long  taskID;
static int            panelHandle;
static short          device;
static unsigned long  numChannels;
static char           channelString[100];
static unsigned short triggerType;
static double         triggerLevel;
static double         waveform[MAX_CHANS*NUM_SCANS];
static double         scanRate;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    panelHandle = LoadPanel (0, "ai^trig.uir", PANEL);
    SetCtrlAttribute (panelHandle, PANEL_ERRORMSG, ATTR_VISIBLE, 0);
    DisplayPanel (panelHandle);
    RunUserInterface ();

    return 0;
}


int CVICALLBACK StartCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int error;
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle, PANEL_DEVICE, &device);
            GetCtrlVal (panelHandle, PANEL_CHANNEL, channelString);
            GetCtrlVal (panelHandle, PANEL_TRIGGER_TYPE, &triggerType);
            if (triggerType == SW_ANALOG_TRIGGER || triggerType == HW_ANALOG_TRIGGER)
                scanRate = 2000.0;
            else
                scanRate = 2000000.0;
            GetCtrlVal (panelHandle, PANEL_TRIGGER_LEVEL, &triggerLevel);
            
            error = GetNumChannels (device, channelString, ANALOG_INPUT, 
                                &numChannels);
            
            SetCtrlAttribute (panelHandle, PANEL_DEVICE, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle, PANEL_CHANNEL, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle, PANEL_TRIGGER_TYPE, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle, PANEL_TRIGGER_LEVEL, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_ENABLED, 1);
            
            break;
    }
    return 0;
}

int CVICALLBACK StopCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int error;
    
    switch (event) {
        case EVENT_COMMIT:
            
            SetCtrlAttribute (panelHandle, PANEL_DEVICE, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle, PANEL_CHANNEL, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_ENABLED, 0);
            SetCtrlAttribute (panelHandle, PANEL_TRIGGER_TYPE, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle, PANEL_TRIGGER_LEVEL, ATTR_DIMMED, 0);
            
            break;
    }
    return 0;
}


int CVICALLBACK TimerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    short error, i;
    
    switch (event) {
        static unsigned long scanBacklog;
        case EVENT_TIMER_TICK:
            error = AIAcquireTriggeredWaveforms (device, channelString, NUM_SCANS,
                                                 scanRate, UPPER_LIMIT,
                                                 LOWER_LIMIT, 0, triggerType, 1,
                                                 triggerLevel, "", 0, -1.0,
                                                 GROUP_BY_CHANNEL, waveform);
        
            if (error == timeOutWarn)
                SetCtrlAttribute (panelHandle, PANEL_ERRORMSG, ATTR_VISIBLE, 1);
            else
                SetCtrlAttribute (panelHandle, PANEL_ERRORMSG, ATTR_VISIBLE, 0);
                    
            
            DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_DELAYED_DRAW);

            for (i=0; i<numChannels; i++)
            {

                PlotY (panelHandle, PANEL_GRAPH, &waveform[i*NUM_SCANS], NUM_SCANS,
                       VAL_DOUBLE, VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1,
                       VAL_GREEN);
            
            }
            
            break;
    }
    return 0;
}


int CVICALLBACK QuitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int error;
    switch (event) {
        case EVENT_COMMIT:
            SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_ENABLED, 0);
            QuitUserInterface (0);
            break;
    }
    return 0;
}


